package gonganxitong

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type order struct {
	Table
	user    user
	event   event
	apply   apply
	order   string `data:"4"`
	role    string `data:"creator"`
	portal  string `data:"true"`
	domain  string `data:"gonganxitong"`
	create  string `name:"create user_uid* apply_uid* status*"`
	reject  string `name:"reject" role:"void"`
	approve string `name:"approve" role:"void"`
}

func (s order) Create(m *ice.Message, arg ...string) {
	s.Table.Create(m, arg...)
	m.Info("what %v", m.FormatChain())
	s.sendTemplate(m, m.Trans("role order create", "权限审批 请处理"))
}
func (s order) List(m *ice.Message, arg ...string) {
	s.Tables(m, s.apply, "left join users on applies.user_uid = users.uid").FieldsWithCreatedAT(m, s.apply,
		model.USER_AVATAR, model.USER_NAME,
		model.PLACE_UID, s.AS(s.Key(s.apply, model.ROLE), s.Keys(s.UserPlace, model.ROLE)), model.ORDER_STATUS,
		model.BEGIN_TIME, model.END_TIME, s.AS(s.Key(s, model.UID), model.ORDER_UID),
	)
	if len(arg) == 1 {
		s.Table.Select(m, s.Key(s, model.USER_UID), m.Option(model.USER_UID), model.PLACE_UID, arg[0])
		defer m.Sort(kit.Fields(model.ORDER_STATUS, model.CREATED_AT), []string{OrderCreate.String(), OrderApproved.String(), OrderRejected.String()}, ice.STR_R)
	} else if len(arg) == 2 {
		s.Table.SelectDetail(m, s.Key(s, model.USER_UID), m.Option(model.USER_UID), model.PLACE_UID, arg[0], s.Key(s.apply, model.UID), arg[1])
	} else {
		return
	}
	s.RenameAppend(m, model.PLACE_UID, s.Keys(s.Place, model.UID)).SelectJoin(m, s.Place, model.NAME, model.TYPE)
	m.Table(func(value ice.Maps) {
		if OrderStatus(kit.Int(value[model.ORDER_STATUS])) == OrderCreate {
			m.PushButton(s.Reject, s.Approve)
		} else {
			m.PushButton()
		}
	}).Action()
	s.UserPlace.RewriteAppend(m)
	s.Display(m, "")
}
func (s order) Reject(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	if s.changeStatus(m, OrderCreate, OrderRejected); m.IsErr() {
		return
	}
	m.Cmdy(m.Prefix("apply"), s.apply.Reject, m.Option(model.UID)).ProcessRefresh()
	s.RecordEvent(m, kit.JoinWord("❌", m.Trans("order rejected", "审批驳回"), s.TransRole(m), m.Option(model.USER_NAME)), m.Option(model.UID))
	s.sendTemplate(m, m.Trans("role order rejected", "权限审批 已驳回"))
}
func (s order) Approve(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	if s.changeStatus(m, OrderCreate, OrderApproved); m.IsErr() {
		return
	}
	m.Cmdy(m.Prefix("apply"), s.apply.Approve, m.Option(model.UID)).ProcessRefresh()
	s.RecordEvent(m, "", kit.JoinWord("✅", m.Trans("order approved", "审批通过"), s.TransRole(m), m.Option(model.USER_NAME)), m.Option(model.UID))
	s.sendTemplate(m, m.Trans("role order approved", "权限审批 已通过"))
}

func init() { ice.TeamCtxCmd(order{Table: newTable()}) }

func (s order) changeStatus(m *ice.Message, from, todo OrderStatus, arg ...string) *ice.Message {
	msg := s.Table.Select(m.Spawn(), model.UID, m.Option(model.ORDER_UID), model.USER_UID, m.Option(model.USER_UID))
	if !m.WarnNotFound(msg.Length() == 0, m.Option(model.UID)) {
		if status := OrderStatus(kit.Int(msg.Append(model.STATUS))); !m.WarnNotValid(status != from, status.String()) {
			s.Table.Update(m, kit.Dict(model.STATUS, todo), model.UID, m.Option(model.ORDER_UID))
		}
	}
	return msg
}
func (s order) sendTemplate(m *ice.Message, info string) *ice.Message {
	s.Table.SendTemplate(m, m.Option(model.USER_NAME), m.Append(model.USER_UID), info)
	return m
}

type OrderStatus int

const (
	OrderCreate OrderStatus = iota
	OrderRejected
	OrderApproved
)

var OrderStatusList = map[OrderStatus]string{
	OrderCreate:   "create",
	OrderRejected: "rejected",
	OrderApproved: "approved",
}

func (s OrderStatus) String() string { return OrderStatusList[s] }
